package com.platform.pos.posprintrabbitmqprovider.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.platform.pos.posprintrabbitmqprovider.common.CommonResult;
import com.platform.pos.posprintrabbitmqprovider.pojo.Message;
import org.springframework.amqp.core.AmqpAdmin;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.FanoutExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.UUID;

@RestController
@RequestMapping("/mqapi")
public class SendMessageController {

    @Autowired
    RabbitTemplate rabbitTemplate;  //使用RabbitTemplate,这提供了接收/发送等等方法
    @Autowired
    AmqpAdmin amqpAdmin;

    @PostMapping("/sendPosPrintDirectMessage")
    public String sendDirectMessage(@RequestBody Message message) {
        message.setMessageId(UUID.randomUUID().toString());
        message.setCreateTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        //将消息携带绑定键值：PosDirectRouting 发送到交换机PosDirectExchange
        rabbitTemplate.convertAndSend("PosDirectExchange", "PosDirectRouting", JSON.toJSONString(message));
        CommonResult commonResult = new CommonResult();
        commonResult.setCode("200");
        commonResult.setMsg("PosDirectExchange发送成功！");
        commonResult.setData(message);
        return JSONObject.toJSONString(commonResult);
    }

    @PostMapping("/sendPosPrintTopicMessageSingle")
    public String sendTopicMessageMan(@RequestBody Message message) {
        message.setMessageId(UUID.randomUUID().toString());
        message.setCreateTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        rabbitTemplate.convertAndSend("topicExchange", "topic.single", JSON.toJSONString(message));
        CommonResult commonResult = new CommonResult();
        commonResult.setCode("200");
        commonResult.setMsg("topicExchangeSingle发送成功！");
        commonResult.setData(message);
        return JSONObject.toJSONString(commonResult);
    }

    @PostMapping("/sendPosPrintTopicMessageAll")
    public String sendTopicMessageAll(@RequestBody Message message) {
        message.setMessageId(UUID.randomUUID().toString());
        message.setCreateTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        rabbitTemplate.convertAndSend("topicExchange", "topic.all", JSON.toJSONString(message));
        CommonResult commonResult = new CommonResult();
        commonResult.setCode("200");
        commonResult.setMsg("topicExchangeAll发送成功！");
        commonResult.setData(message);
        return JSONObject.toJSONString(commonResult);
    }

    @PostMapping("/sendPosPrintFanoutMessage")
    public String sendFanoutMessage(@RequestBody Message message) {
        //创建扇形交换机
        amqpAdmin.declareExchange(new FanoutExchange(message.getPosFanoutExchange()));
        //创建消息队列
        amqpAdmin.declareQueue(new Queue(message.getPosQueue(), true));
        //将消息队列绑定到交换机
        amqpAdmin.declareBinding(new Binding(message.getPosQueue(), Binding.DestinationType.QUEUE, message.getPosFanoutExchange(), "FanoutRoutting", null));
        message.setMessageId(UUID.randomUUID().toString());
        message.setCreateTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        rabbitTemplate.convertAndSend(message.getPosFanoutExchange(), null, JSON.toJSONString(message));
        CommonResult commonResult = new CommonResult();
        commonResult.setCode("200");
        commonResult.setMsg(message.getPosFanoutExchange()+"发送成功！");
        commonResult.setData(message);
        return JSONObject.toJSONString(commonResult);
    }
}
